# -*- coding: utf-8 -*-

from nose.tools.trivial import (
    assert_is_none,
    assert_is_not_none,
    assert_is_instance,
    assert_equals
)

from boto.s3.connection import S3Connection

from assetman.tests import (
    TEST_KEY,
    TEST_SECRET,
    TEST_BUCKET
)

from assetman.core import Assetman
from assetman.core.assets import Asset


def test_assetman():
    a = Assetman(TEST_KEY, TEST_SECRET, TEST_BUCKET)
    assert_is_none(a._connection)
    ctx = a.connection
    assert_is_instance(ctx, S3Connection)

    ''' Simple Asset '''

    asset = Asset('my sweet data')
    a.save_asset(asset)
    assert_is_not_none(asset.uuid)
    retrieved = a.get_asset(asset.uuid)
    assert_equals(asset.uuid, retrieved.uuid)
    a.delete_asset(asset.uuid)
    assert_is_none(a.get_asset(asset.uuid))

    ''' Prefixed Asset '''

    asset = Asset('My Prefixed Data', prefix='mah_prefix/bro')
    a.save_asset(asset)
    assert_is_not_none(asset.uuid)
    retrieved = a.get_asset(asset.uuid, asset.prefix)
    assert_equals(asset.uuid, retrieved.uuid)
    a.delete_asset(asset.uuid, asset.prefix)
    assert_is_none(a.get_asset(asset.uuid, asset.prefix))



# -*- coding: utf-8 -*-

from boto.s3.connection import S3Connection
from boto.s3.key import Key


def get_connection(key, secret):
    connection = S3Connection(key, secret)
    return connection


def get_bucket(bucket_name, connection):
    bucket = connection.get_bucket(bucket_name)
    return bucket


def upload_string_data(bucket, path, string_data, public=False,
                       mime_type=None, metadata=None):

    key = Key(bucket, path)
    if mime_type:
        key.content_type = mime_type
    if metadata:
        key.update_metadata(metadata)
    key.set_contents_from_string(string_data)
    if public:
        key.make_public()


def delete_key(bucket, path):
    key = bucket.get_key(path)
    if key:
        key.delete()


def get_key(bucket, path):
    key = bucket.get_key(path)
    return key


def get_string_data(bucket, path):
    key = bucket.get_key(path)
    if key:
        return key.get_contents_as_string()



# -*- coding: utf-8 -*-

import uuid


def lookup_by_type(asset_type):
    _asset_type_map = {
        'image': Image
    }

    try:
        return _asset_type_map[asset_type]
    except KeyError:
        pass

    return Asset


class Asset(object):

    def __init__(self, data=None, uuid=None, prefix=None):
        self.data = data
        self.mime_type = None
        self.meta_data = {}
        self.public = False
        self._prefix = prefix
        self._uuid = uuid

    @property
    def uuid(self):
        if not self._uuid:
            self._uuid = uuid.uuid4()
        return self._uuid

    @property
    def asset_type(self):
        return self.meta_data['asset_type']

    @property
    def prefix(self):
        return self._prefix

    def move(self, new_prefix):
        raise NotImplementedError('Support for moving assets is coming soon!')


class Image(Asset):

    def __init__(self, data, height=0, width=0, mimetype=None):
        super(Image, self).__init__(data)
        self.mime_type = mimetype
        self.meta_data['height'] = height
        self.meta_data['width'] = width
        self.meta_data['asset_type'] = 'image'

    @property
    def height(self):
        return self.meta_data['height']

    @property
    def width(self):
        return self.meta_data['width']



# -*- coding: utf-8 -*-

from assetman.core import Assetman
from assetman.core.assets import (
    Asset,
    Image
)


def init(key, secret, bucket):
    return Assetman(key, secret, bucket)


def create_blob(data):
    return Asset(data)


def create_image(data, height, width, mimetype):
    return Image(data, height, width, mimetype)



# -*- coding: utf-8 -*-

from nose.tools.trivial import (
    assert_is_instance
)

from os import path
from wand.image import Image as WandImage

from assetman import (
    init,
    create_blob,
    create_image
)

from assetman.core import Assetman
from assetman.core.assets import (
    Asset,
    Image
)

from assetman.tests import (
    TEST_KEY,
    TEST_SECRET,
    TEST_BUCKET
)


def test_init():
    am = init(TEST_KEY, TEST_SECRET, TEST_BUCKET)
    assert_is_instance(am, Assetman)


def test_create_blob():
    b = create_blob('some random data')
    assert_is_instance(b, Asset)


def test_create_image():
    with WandImage(
        filename=path.join(path.dirname(__file__),
                           'fixtures/faceoff.jpg')) as image:
        i = create_image(
            image.make_blob(), image.height, image.width, image.mimetype)
        assert_is_instance(i, Image)



# -*- coding: utf-8 -*-

from assetman.service import s3
from assetman.core import assets


class Assetman(object):

    def __init__(self, key, secret, bucket):
        self._aws_key = key
        self._aws_secret = secret
        self._bucket_name = bucket
        self._connection = None

    @property
    def connection(self):
        if not self._connection:
            self._connection = s3.get_connection(
                self._aws_key, self._aws_secret)
        return self._connection

    def save_asset(self, asset):
        bucket = s3.get_bucket(self._bucket_name, self.connection)
        s3.upload_string_data(
            bucket,
            (asset.prefix or u'') + u'/' + unicode(asset.uuid),
            asset.data,
            asset.public,
            asset.mime_type,
            asset.meta_data)
        return asset

    def get_asset(self, uuid, prefix=None):
        bucket = s3.get_bucket(self._bucket_name, self.connection)
        key = s3.get_key(bucket, (prefix or u'') + u'/' + unicode(uuid))
        if not key:
            return None

        asset = assets.lookup_by_type(key.get_metadata('asset_type'))(
            key.get_contents_as_string(), uuid)
        asset.url = key.generate_url(120)
        asset.mime_type = key.content_type
        asset.meta_data = key.metadata
        asset._prefix = prefix
        return asset

    def delete_asset(self, uuid, prefix=None):
        bucket = s3.get_bucket(self._bucket_name, self.connection)
        s3.delete_key(
            bucket,
            (prefix or u'') + u'/' + unicode(uuid)
        )
